import geopandas
import pandas as pd
from shapely.geometry import Point

from .data import cat


def to_geodataframe(
    df: pd.DataFrame, lat_key: str = 'LAT', lon_key: str = 'LON'
) -> geopandas.GeoDataFrame:
    ''' helper function to covert DataFrame to GeoDataFrame '''
    geo_df = geopandas.GeoDataFrame(
        df, crs='epsg:4326', geometry=[Point(xy) for xy in zip(df[lon_key], df[lat_key])]
    )
    return geo_df


def load_arb_fortypcds():
    '''load map between assessment area code and its associated fortypcds'''
    d = cat.arb_fortypcds.read()[0]
    return {int(k): v for k, v in d.items()}


def load_ss_codes() -> dict:
    ss_cd_map = cat.super_section_lookup.read()
    return ss_cd_map.set_index("ss")["code"].to_dict()


def load_aa_codes() -> dict:
    aa_code_map = cat.assessment_area_lookup.read()

    return aa_code_map.set_index("aa")["code"].to_dict()


def aa_code_to_ss_code():
    return {
        1.0: 1,
        2.0: 1,
        3.0: 2,
        4.0: 2,
        5.0: 2,
        6.0: 2,
        7.0: 2,
        8.0: 2,
        9.0: 2,
        10.0: 3,
        11.0: 3,
        12.0: 3,
        13.0: 3,
        14.0: 3,
        15.0: 3,
        16.0: 4,
        17.0: 4,
        18.0: 4,
        19.0: 4,
        20.0: 4,
        21.0: 4,
        22.0: 5,
        23.0: 5,
        24.0: 5,
        25.0: 6,
        26.0: 6,
        27.0: 6,
        28.0: 7,
        29.0: 7,
        30.0: 7,
        31.0: 7,
        32.0: 7,
        33.0: 8,
        34.0: 9,
        35.0: 10,
        36.0: 10,
        37.0: 11,
        38.0: 11,
        39.0: 12,
        40.0: 12,
        41.0: 13,
        42.0: 13,
        43.0: 13,
        44.0: 13,
        45.0: 13,
        46.0: 14,
        47.0: 14,
        48.0: 14,
        49.0: 14,
        50.0: 14,
        51.0: 14,
        52.0: 14,
        53.0: 15,
        54.0: 15,
        55.0: 15,
        56.0: 15,
        57.0: 16,
        58.0: 16,
        59.0: 16,
        60.0: 16,
        61.0: 16,
        62.0: 16,
        63.0: 16,
        64.0: 17,
        65.0: 17,
        66.0: 17,
        67.0: 17,
        68.0: 17,
        69.0: 17,
        70.0: 18,
        71.0: 18,
        72.0: 19,
        73.0: 19,
        74.0: 19,
        75.0: 20,
        76.0: 20,
        77.0: 20,
        78.0: 20,
        79.0: 21,
        80.0: 21,
        81.0: 21,
        82.0: 22,
        83.0: 23,
        84.0: 23,
        85.0: 23,
        86.0: 24,
        87.0: 24,
        88.0: 24,
        89.0: 24,
        90.0: 24,
        91.0: 24,
        92.0: 24,
        93.0: 25,
        94.0: 25,
        95.0: 26,
        96.0: 26,
        97.0: 27,
        98.0: 27,
        99.0: 27,
        100.0: 27,
        101.0: 27,
        102.0: 28,
        103.0: 28,
        104.0: 28,
        105.0: 28,
        106.0: 28,
        107.0: 28,
        108.0: 29,
        109.0: 29,
        110.0: 30,
        111.0: 31,
        112.0: 32,
        113.0: 32,
        114.0: 32,
        115.0: 32,
        116.0: 32,
        117.0: 33,
        118.0: 33,
        119.0: 33,
        120.0: 33,
        121.0: 33,
        122.0: 34,
        123.0: 34,
        124.0: 34,
        125.0: 34,
        126.0: 34,
        127.0: 34,
        128.0: 34,
        129.0: 35,
        130.0: 35,
        131.0: 35,
        132.0: 35,
        133.0: 35,
        155.0: 40,
        156.0: 40,
        157.0: 40,
        158.0: 40,
        159.0: 40,
        160.0: 40,
        161.0: 40,
        141.0: 37,
        142.0: 37,
        143.0: 37,
        144.0: 37,
        145.0: 37,
        146.0: 37,
        147.0: 37,
        148.0: 38,
        149.0: 38,
        150.0: 38,
        151.0: 39,
        152.0: 39,
        153.0: 39,
        154.0: 39,
        162.0: 41,
        163.0: 41,
        164.0: 41,
        165.0: 41,
        166.0: 41,
        167.0: 42,
        168.0: 42,
        169.0: 42,
        170.0: 42,
        171.0: 42,
        172.0: 42,
        173.0: 43,
        174.0: 43,
        175.0: 44,
        176.0: 44,
        177.0: 44,
        178.0: 45,
        179.0: 45,
        180.0: 46,
        181.0: 46,
        182.0: 46,
        183.0: 46,
        184.0: 46,
        185.0: 47,
        186.0: 47,
        187.0: 47,
        188.0: 47,
        189.0: 48,
        190.0: 48,
        191.0: 48,
        192.0: 48,
        193.0: 48,
        194.0: 48,
        195.0: 49,
        196.0: 49,
        197.0: 49,
        198.0: 49,
        199.0: 49,
        200.0: 49,
        201.0: 50,
        202.0: 50,
        203.0: 50,
        204.0: 50,
        205.0: 50,
        206.0: 51,
        207.0: 52,
        208.0: 52,
        209.0: 52,
        210.0: 53,
        211.0: 53,
        212.0: 53,
        213.0: 53,
        214.0: 53,
        215.0: 54,
        216.0: 54,
        217.0: 54,
        218.0: 54,
        219.0: 54,
        220.0: 54,
        221.0: 55,
        222.0: 55,
        223.0: 56,
        224.0: 56,
        225.0: 57,
        226.0: 57,
        227.0: 57,
        228.0: 58,
        229.0: 58,
        230.0: 59,
        231.0: 60,
        232.0: 60,
        233.0: 60,
        234.0: 61,
        235.0: 62,
        236.0: 62,
        237.0: 62,
        238.0: 62,
        239.0: 62,
        240.0: 63,
        241.0: 63,
        242.0: 63,
        243.0: 63,
        244.0: 63,
        245.0: 64,
        246.0: 64,
        247.0: 64,
        248.0: 64,
        249.0: 64,
        250.0: 65,
        251.0: 65,
        252.0: 65,
        253.0: 65,
        254.0: 66,
        255.0: 66,
        256.0: 66,
        257.0: 67,
        258.0: 68,
        259.0: 68,
        260.0: 68,
        261.0: 68,
        262.0: 68,
        263.0: 68,
        264.0: 69,
        265.0: 69,
        266.0: 69,
        267.0: 69,
        268.0: 70,
        269.0: 70,
        270.0: 70,
        271.0: 70,
        272.0: 70,
        273.0: 71,
        274.0: 71,
        275.0: 71,
        276.0: 71,
        277.0: 71,
        278.0: 71,
        279.0: 72,
        280.0: 72,
        281.0: 72,
        282.0: 72,
        283.0: 73,
        284.0: 74,
        285.0: 285,
        286.0: 286,
        287.0: 287,
        288.0: 76,
        289.0: 76,
        290.0: 76,
        291.0: 76,
        292.0: 76,
        293.0: 77,
        294.0: 78,
        295.0: 78,
        296.0: 79,
        297.0: 79,
        298.0: 79,
        299.0: 80,
        300.0: 80,
        301.0: 80,
        302.0: 81,
        303.0: 81,
        304.0: 81,
        305.0: 82,
        306.0: 82,
        307.0: 82,
        308.0: 83,
        309.0: 83,
        310.0: 83,
        311.0: 83,
        312.0: 84,
        313.0: 85,
        314.0: 85,
        315.0: 85,
        316.0: 85,
        317.0: 86,
        318.0: 86,
        319.0: 86,
        320.0: 87,
        321.0: 87,
        322.0: 87,
        323.0: 87,
        324.0: 87,
        325.0: 87,
        326.0: 88,
        327.0: 88,
        328.0: 88,
        329.0: 88,
        330.0: 89,
        331.0: 89,
        332.0: 89,
        333.0: 89,
        334.0: 89,
        335.0: 90,
        336.0: 90,
        337.0: 90,
        338.0: 90,
        339.0: 90,
        340.0: 91,
        341.0: 91,
        342.0: 91,
        343.0: 92,
        344.0: 93,
        345.0: 93,
        346.0: 94,
        347.0: 94,
        348.0: 94,
        349.0: 94,
        350.0: 95,
        351.0: 95,
        352.0: 95,
        353.0: 95,
        354.0: 96,
        355.0: 97,
        356.0: 97,
        357.0: 97,
        358.0: 97,
    }


def supersection_str_to_ss_code():
    # from original ancillary files, handrolled by G Badgley
    sseciton_str_to_code = {
        'Adirondacks & Green Mountains': 1,
        'Allegheny & North Cumberland Mountains': 2,
        'Aroostook Hills and Lowlands': 3,
        'Atlantic Coastal Plain & Flatwoods': 4,
        'Bitterroot Mountains': 5,
        'Blue Mountains': 6,
        'Blue Ridge Mountains': 7,
        'Booneville Basin': 8,
        'California Central Valley Basin': 9,
        'Catskill Mountains': 10,
        'Central California Coast': 11,
        'Central Great Plains': 12,
        'Central Interior Broadleaf Forest Central Till Plains': 13,
        'Central Interior Broadleaf Forest Eastern Low Plateau': 14,
        'Central Interior Broadleaf Forest Ozark Highlands': 15,
        'Central Interior Broadleaf Forest Western Low Plateau': 16,
        'Central Maine & Fundy Coast & Ebayment': 17,
        'Central New Mexico': 18,
        'Chihuahuan Semi-Desert': 19,
        'Colorado Plateau': 20,
        'Colorado River Canyon Lands': 21,
        'Columbia Basin': 22,
        'Cross Timbers and Prairie': 23,
        'Eastern Broadleaf Forest Cumberland Plateau & Valley': 24,
        'Eastern Cascades': 25,
        'Eastern Great Plains': 26,
        'Erie & Ontario Lake Plain': 27,
        'Florida Coastal Plains Central Highlands': 28,
        'Florida Everglades': 29,
        'Great Divide Basin': 30,
        'Great Plains': 31,
        'Gulf Coastal Plain': 32,
        'Idaho Batholith': 33,
        'Laurentian Mixed Forest Arrowhead': 34,
        'Laurentian Mixed Forest Green Bay Lobe': 35,
        'Laurentian Mixed Forest MN & Ontario Lake Plain': 36,
        'Laurentian Mixed Forest NLP/EUP': 37,
        'Laurentian Mixed Forest Northern Highlands': 38,
        'Laurentian Mixed Forest Southern Superior': 39,
        'Laurentian Mixed Forest Western Superior & Lake Plains': 40,
        'Lower New England - Northern Appalachia': 41,
        'Maine - New Brunswick Foothills and Lowlands': 42,
        'Modoc Plateau': 43,
        'Montana Rocky Mountains': 44,
        'MS River Delta': 45,
        'MS River Mixed Forest': 46,
        'MW Broadleaf Forest Central Till Plains': 47,
        'MW Broadleaf Forest Driftless & Morainal': 48,
        'MW Broadleaf Forest Great Lakes Morainal & Sands': 49,
        'MW Broadleaf Forest SC Great Lakes & Lake Whittlesey': 50,
        'Nevada Mountains': 51,
        'North Central Great Plains': 52,
        'Northern Allegheny Plateau': 53,
        'Northern Atlantic Coastal Plain': 54,
        'Northern California Coast': 55,
        'Northern Great Plains': 56,
        'Northern Rocky Mountains': 57,
        'Northwest Cascades': 58,
        'Northwestern Basin and Range': 59,
        'Okanogan Highland': 60,
        'Oregon and Washington Coast': 61,
        'Ouachita Mixed Forest': 62,
        'Ozark Broadleaf Forest-Meadow Boston Mountains': 63,
        'Prairie Parkland Central Till Plains & Grand Prairies': 64,
        'Prairie Parkland North Central Plains': 65,
        'Prairie Parkland Red River Valley': 66,
        'Puget Trough': 67,
        'SE Middle Mixed Forest Arkansas Valley': 68,
        'SE Middle Mixed Forest Cumberland Plateau & Valley': 69,
        'SE Middle Mixed Forest Piedmont': 70,
        'SE Middle Mixed Forest Western Mid Coastal Plains': 71,
        'Sierra Nevada': 72,
        'Sierra Nevada Foothills': 73,
        'Snake River Basin': 74,
        'Southeast and South Central Alaska': 75,
        'Southern Allegheny Plateau': 76,
        'Southern California Coast': 77,
        'Southern California Mountains': 78,
        'Southern Cascades': 79,
        'Southern Rockies Front Range': 80,
        'Southern Rocky Mountains': 81,
        'Southwest High Plains': 82,
        'Southwest Plateau': 83,
        'Southwestern Desert': 84,
        'Southwestern Rocky Mountains': 85,
        'St Lawrence & Mohawk Valley': 86,
        'Subtropical Prairie Parkland Gulf & Oak Prairie': 87,
        'Tonto Transition': 88,
        'Utah Mountains': 89,
        'Wasatch Range': 90,
        'Western Allegheny Plateau': 91,
        'Western Basin and Range': 92,
        'Western Great Plains': 93,
        'White Mountains': 94,
        'White Mountains - San Francisco Peaks - Mongollon': 95,
        'Willamette Valley': 96,
        'Yellowstone / Bighorn': 97,
    }

    return sseciton_str_to_code


def assessment_area_str_to_aa_code():
    return {
        'Adirondacks & Green Mountains Northeast Conifers': 1,
        'Adirondacks & Green Mountains Northern Hardwood': 2,
        'Allegheny & North Cumberland Mountains Cove Forests': 3,
        'Allegheny & North Cumberland Mountains Mixed Hardwoods': 4,
        'Allegheny & North Cumberland Mountains Mixed Pine-Hardwood': 5,
        'Allegheny & North Cumberland Mountains Northern Conifer': 6,
        'Allegheny & North Cumberland Mountains Northern Hardwoods': 7,
        'Allegheny & North Cumberland Mountains Oak-Hickory': 8,
        'Allegheny & North Cumberland Mountains Pine': 9,
        'Aroostook Hills and Lowlands Conifer Bog': 10,
        'Aroostook Hills and Lowlands Lowland Hardwoods': 11,
        'Aroostook Hills and Lowlands Northern Hardwoods': 12,
        'Aroostook Hills and Lowlands Pine Forests': 13,
        'Aroostook Hills and Lowlands Poplar/Birch': 14,
        'Aroostook Hills and Lowlands Spruce-Fir': 15,
        'Atlantic Coastal Plain Loblolly-Shortleaf-Oak': 16,
        'Atlantic Coastal Plain Longleaf-Slash Pine': 17,
        'Atlantic Coastal Plain Oak-Hickory': 18,
        'Atlantic Coastal Plain Riverine Hardwood': 19,
        'Atlantic Coastal Plain Swamp Hardwood & Cypress': 20,
        'Atlantic Coastal Plain Upland Hardwood': 21,
        'Bitterroot Fir / Spruce / Hemlock': 22,
        'Bitterroot Mixed Conifer': 23,
        'Bitterroot Subalpine Mixed Conifer': 24,
        'Blue Mountain High Elevation Conifer': 25,
        'Blue Mountain Juniper / Pinyon Woodland': 26,
        'Blue Mountain Mixed Conifer': 27,
        'Blue Ridge Cove Forest': 28,
        'Blue Ridge Mixed Hardwoods': 29,
        'Blue Ridge Northern Hardwoods': 30,
        'Blue Ridge Oak-Hickory': 31,
        'Blue Ridge Oak-Pine': 32,
        'Bonneville Basin Pinyon / Juniper Mixed Woodland': 33,
        'California Valley Oak Woodland': 34,
        'Catskill Mountains Northeast Conifer-Oak-Hickory': 35,
        'Catskill Mountains Northern Hardwood': 36,
        'Central California Coast Redwood/Douglas-fir Mixed Conifer': 37,
        'Central California Coast Mixed Oak Woodland': 38,
        'Central Great Plains Mixed Forest': 39,
        'Central Great Plains Riparian Hardwood': 40,
        'Central Interior Broadleaf Forest Central Till Plains Lowland Hardwoods': 41,
        'Central Interior Broadleaf Forest Central Till Plains Mixed Upland Hardwoods': 42,
        'Central Interior Broadleaf Forest Central Till Plains Northern Hardwoods': 43,
        'Central Interior Broadleaf Forest Central Till Plains Oak-Hickory': 44,
        'Central Interior Broadleaf Forest Central Till Plains Oak-Pine': 45,
        'Central Interior Broadleaf Forest Eastern Low Plateau Cove Forest': 46,
        'Central Interior Broadleaf Forest Eastern Low Plateau Lowland Hardwoods': 47,
        'Central Interior Broadleaf Forest Eastern Low Plateau Mixed Upland Hardwoods': 48,
        'Central Interior Broadleaf Forest Eastern Low Plateau Northern Hardwoods': 49,
        'Central Interior Broadleaf Forest Eastern Low Plateau Oak-Hickory': 50,
        'Central Interior Broadleaf Forest Eastern Low Plateau Oak-Pine': 51,
        'Central Interior Broadleaf Forest Eastern Low Plateau Pine': 52,
        'Central Interior Broadleaf Forest Ozark Highlands Lowland Hardwoods': 53,
        'Central Interior Broadleaf Forest Ozark Highlands Mixed Upland Hardwoods': 54,
        'Central Interior Broadleaf Forest Ozark Highlands Oak-Hickory': 55,
        'Central Interior Broadleaf Forest Ozark Highlands Oak-Pine': 56,
        'Central Interior Broadleaf Forest Western Low Plateau Cove Forest': 57,
        'Central Interior Broadleaf Forest Western Low Plateau Lowland Hardwoods': 58,
        'Central Interior Broadleaf Forest Western Low Plateau Mixed Upland Hardwoods': 59,
        'Central Interior Broadleaf Forest Western Low Plateau Northern Hardwoods': 60,
        'Central Interior Broadleaf Forest Western Low Plateau Oak-Hickory': 61,
        'Central Interior Broadleaf Forest Western Low Plateau Oak-Pine': 62,
        'Central Interior Broadleaf Forest Western Low Plateau Pine': 63,
        'Central Maine & Fundy Coast & Ebayment Conifer Bog': 64,
        'Central Maine & Fundy Coast & Ebayment Lowland Hardwoods': 65,
        'Central Maine & Fundy Coast & Ebayment Northern Hardwoods': 66,
        'Central Maine & Fundy Coast & Ebayment Pine Forests': 67,
        'Central Maine & Fundy Coast & Ebayment Poplar/Birch': 68,
        'Central Maine & Fundy Coast & Ebayment Spruce-Fir': 69,
        'Central New Mexico Mixed Conifer': 70,
        'Central New Mexico Pinyon / Juniper Woodland': 71,
        'Chihuahuan Mixed Oak Woodland': 72,
        'Chihuahuan Pinyon / Juniper Woodland': 73,
        'Chihuahuan Pinyon / Mesquite Woodland': 74,
        'Colorado Plateau High Elevation Conifer': 75,
        'Colorado Plateau Oak Woodland': 76,
        'Colorado Plateau Pinyon/ Juniper Woodland': 77,
        'Colorado Plateau Ponderosa Pine': 78,
        'Colorado River Aspen': 79,
        'Colorado River Mixed Forest': 80,
        'Colorado River Pinyon / Juniper Woodlands': 81,
        'Columbia Basin Mixed Conifer': 82,
        'Cross Timbers and Prairie Lowland Hardwood': 83,
        'Cross Timbers and Prairie Oak / Hickory': 84,
        'Cross Timbers and Prairie Oak / Pine / Cedar': 85,
        'Eastern Broadleaf Forest Cumberland Plateau & Valley Cove Forest': 86,
        'Eastern Broadleaf Forest Cumberland Plateau & Valley Lowland Hardwoods': 87,
        'Eastern Broadleaf Forest Cumberland Plateau & Valley Mixed Upland Hardwoods': 88,
        'Eastern Broadleaf Forest Cumberland Plateau & Valley Northern Hardwoods': 89,
        'Eastern Broadleaf Forest Cumberland Plateau & Valley Oak-Hickory': 90,
        'Eastern Broadleaf Forest Cumberland Plateau & Valley Oak-Pine': 91,
        'Eastern Broadleaf Forest Cumberland Plateau & Valley Pine': 92,
        'Eastern Cascade High Elevation Conifer': 93,
        'Eastern Cascade Mixed Conifer': 94,
        'Eastern Great Plains Mixed Woodland': 95,
        'Eastern Great Plains Riverine Hardwoods': 96,
        'Erie & Ontario Lake Plain Aspen-Birch': 97,
        'Erie & Ontario Lake Plain Elm-Ash-Red Maple': 98,
        'Erie & Ontario Lake Plain Northeast Conifer': 99,
        'Erie & Ontario Lake Plain Northern Hardwood': 100,
        'Erie & Ontario Lake Plain Oak-Hickory': 101,
        'Florida Coastal Plains Central Highlands Hardwood & Cypress': 102,
        'Florida Coastal Plains Central Highlands Loblolly-Shortleaf-Oak': 103,
        'Florida Coastal Plains Central Highlands Longleaf-Slash Pine': 104,
        'Florida Coastal Plains Central Highlands Mixed Upland Hardwood': 105,
        'Florida Coastal Plains Central Highlands Oak-Hickory': 106,
        'Florida Coastal Plains Central Highlands Riverine Hardwood': 107,
        'Florida Everglades Swamp & River Hardwood & Cypress': 108,
        'Florida Everglades Woodland Islands': 109,
        'Great Divide Basin Woodland': 110,
        'Great Plains Woodland': 111,
        'Gulf Coastal Plain Cypress & Swamp Hardwood': 112,
        'Gulf Coastal Plain Loblolly-Shortleaf-Oak': 113,
        'Gulf Coastal Plain Longleaf-Slash Pine': 114,
        'Gulf Coastal Plain Mixed Hardwoods': 115,
        'Gulf Coastal Plain Riverine Hardwood': 116,
        'Idaho Batholith Douglas-fir / Ponderosa': 117,
        'Idaho Batholith Lodgepole Pine': 118,
        'Idaho Batholith Mixed Conifer': 119,
        'Idaho Batholith Ponderosa Pine': 120,
        'Idaho Batholith Spruce / Fir': 121,
        'Laurentian Mixed Forest Arrowhead Conifer Bog': 122,
        'Laurentian Mixed Forest Arrowhead Lowland Hardwoods': 123,
        'Laurentian Mixed Forest Arrowhead Northern Hardwoods': 124,
        'Laurentian Mixed Forest Arrowhead Oak-Hickory': 125,
        'Laurentian Mixed Forest Arrowhead Pine Forests': 126,
        'Laurentian Mixed Forest Arrowhead Poplar/Birch': 127,
        'Laurentian Mixed Forest Arrowhead Spruce-Fir': 128,
        'Laurentian Mixed Forest Green Bay Lobe Conifer Bog': 129,
        'Laurentian Mixed Forest Green Bay Lobe Lowland Hardwoods': 130,
        'Laurentian Mixed Forest Green Bay Lobe Northern Hardwoods': 131,
        'Laurentian Mixed Forest Green Bay Lobe Pine-Oak-Hickory': 132,
        'Laurentian Mixed Forest Green Bay Lobe Poplar-Spruce-Fir': 133,
        'Laurentian Mixed Forest Western Superior & Lake Plains Conifer Bog': 155,
        'Laurentian Mixed Forest Western Superior & Lake Plains Lowland Hardwoods': 156,
        'Laurentian Mixed Forest Western Superior & Lake Plains Maple-Beech-Birch': 157,
        'Laurentian Mixed Forest Western Superior & Lake Plains Oak-Hickory': 158,
        'Laurentian Mixed Forest Western Superior & Lake Plains Pine Forests': 159,
        'Laurentian Mixed Forest Western Superior & Lake Plains Poplar/Birch': 160,
        'Laurentian Mixed Forest Western Superior & Lake Plains Spruce-Fir': 161,
        'Laurentian Mixed Forest NLP/EUP Conifer Bog': 141,
        'Laurentian Mixed Forest NLP/EUP Lowland Hardwoods': 142,
        'Laurentian Mixed Forest NLP/EUP Northern Hardwoods': 143,
        'Laurentian Mixed Forest NLP/EUP Oak-Hickory': 144,
        'Laurentian Mixed Forest NLP/EUP Pine Forests': 145,
        'Laurentian Mixed Forest NLP/EUP Poplar/Birch': 146,
        'Laurentian Mixed Forest NLP/EUP Spruce-Fir': 147,
        'Laurentian Mixed Forest Northern Highlands Conifer Bog': 148,
        'Laurentian Mixed Forest Northern Highlands Mixed Northern Hardwoods': 149,
        'Laurentian Mixed Forest Northern Highlands Mixed Pine-Oak': 150,
        'Laurentian Mixed Forest Southern Superior Conifer Bog': 151,
        'Laurentian Mixed Forest Southern Superior Maple-Birch-Beech': 152,
        'Laurentian Mixed Forest Southern Superior Pine-Oak-Hickory': 153,
        'Laurentian Mixed Forest Southern Superior Poplar-Spruce-Fir': 154,
        'Lower New England - Northern Appalachia Mixed Hardwood': 162,
        'Lower New England - Northern Appalachia Northern Conifer': 163,
        'Lower New England - Northern Appalachia Northern Hardwood': 164,
        'Lower New England - Northern Appalachia Oak-Hickory': 165,
        'Lower New England - Northern Appalachia Shortleaf-Loblolly-Oak': 166,
        'Maine - New Brunswick Foothills and Lowlands Conifer Bog': 167,
        'Maine - New Brunswick Foothills and Lowlands Lowland Hardwoods': 168,
        'Maine - New Brunswick Foothills and Lowlands Northern Hardwoods': 169,
        'Maine - New Brunswick Foothills and Lowlands Pine Forests': 170,
        'Maine - New Brunswick Foothills and Lowlands Poplar/Birch': 171,
        'Maine - New Brunswick Foothills and Lowlands Spruce-Fir': 172,
        'Modoc High Elevation Mixed Conifer': 173,
        'Modoc Mixed Conifer': 174,
        'Montana Rockies Lodgepole Pine': 175,
        'Montana Rockies Mixed Conifer': 176,
        'Montana Rockies Spruce / Fir': 177,
        'MS River Delta Hardwood': 178,
        'MS River Delta Woodland Islands': 179,
        'MS River Mixed Forest Coastal Plain Mixed Hardwood-Pine': 180,
        'MS River Mixed Forest Coastal Plain Oak-Hickory': 181,
        'MS River Mixed Forest Coastal Plain Riverine Hardwood': 182,
        'MS River Mixed Forest Coastal Plain Southern Pine': 183,
        'MS River Mixed Forest Coastal Plain Swamp Hardwood': 184,
        'MW Broadleaf Forest Central Till Central Lowland Hardwoods': 185,
        'MW Broadleaf Forest Central Till Central Maple-Beech-Birch': 186,
        'MW Broadleaf Forest Central Till Central Oak-Hickory': 187,
        'MW Broadleaf Forest Central Till Central Pine Forests': 188,
        'MW Broadleaf Forest Driftless & Morainal Lowland Hardwoods': 189,
        'MW Broadleaf Forest Driftless & Morainal Maple-Beech-Birch': 190,
        'MW Broadleaf Forest Driftless & Morainal Oak-Hickory': 191,
        'MW Broadleaf Forest Driftless & Morainal Pine Forests': 192,
        'MW Broadleaf Forest Driftless & Morainal Poplar/Birch': 193,
        'MW Broadleaf Forest Driftless & Morainal Spruce-Fir': 194,
        'MW Broadleaf Forest Great Lakes Morainal & Sands Conifer Bog': 195,
        'MW Broadleaf Forest Great Lakes Morainal & Sands Lowland Hardwoods': 196,
        'MW Broadleaf Forest Great Lakes Morainal & Sands Maple-Beech-Birch': 197,
        'MW Broadleaf Forest Great Lakes Morainal & Sands Oak-Hickory': 198,
        'MW Broadleaf Forest Great Lakes Morainal & Sands Pine Forests': 199,
        'MW Broadleaf Forest Great Lakes Morainal & Sands Poplar/Birch': 200,
        'MW Broadleaf Forest SC Great Lakes & Lake Whittlesey Conifer Forest': 201,
        'MW Broadleaf Forest SC Great Lakes & Lake Whittlesey Lowland Hardwoods': 202,
        'MW Broadleaf Forest SC Great Lakes & Lake Whittlesey Maple-Birch-Beech': 203,
        'MW Broadleaf Forest SC Great Lakes & Lake Whittlesey Oak-Hickory': 204,
        'MW Broadleaf Forest SC Great Lakes & Lake Whittlesey Poplar/Birch': 205,
        'Nevada Mountains Pinyon/ Juniper Mixed Woodlands': 206,
        'North Central Great Plains Aspen': 207,
        'North Central Great Plains Oak Woodland': 208,
        'North Central Great Plains Riparian Hardwood': 209,
        'Northern Allegheny Plateau Aspen-Birch': 210,
        'Northern Allegheny Plateau Northern Conifer': 211,
        'Northern Allegheny Plateau Northern Hardwood': 212,
        'Northern Allegheny Plateau Oak-Hickory': 213,
        'Northern Allegheny Plateau Pine-Hardwood': 214,
        'Northern Atlantic Coastal Plain Mixed Hardwood': 215,
        'Northern Atlantic Coastal Plain Oak-Hickory': 216,
        'Northern Atlantic Coastal Plain Oak-Pine': 217,
        'Northern Atlantic Coastal Plain Riverine Hardwood': 218,
        'Northern Atlantic Coastal Plain Southern Pine': 219,
        'Northern Atlantic Coastal Plain Swamp Hardwood & Cypress': 220,
        'Northern California Coast Redwood/Douglas-fir Mixed Conifer': 221,
        'Northern California Coast Mixed Oak Woodland': 222,
        'Northern Great Plains Hardwoods': 223,
        'Northern Great Plains Ponderosa Pine': 224,
        'Northern Rockies Mixed Conifer': 225,
        'Northern Rockies Spruce / Fir': 226,
        'Northern Rockies Subalpine Mixed Conifer': 227,
        'Northwest Cascade High Elevation Conifer': 228,
        'Northwest Cascade Mixed Conifer': 229,
        'Northwest Basin and Range Mixed Conifer': 230,
        'Okanogan Douglas-fir Mixed Conifer': 231,
        'Okanogan Ponderosa': 232,
        'Okanogan Subalpine Mixed Conifer': 233,
        'Northwest Coast Range Forest': 234,
        'Ouachita Mixed Forest-Meadow Ouachita Mountains Loblolly-Shortleaf': 235,
        'Ouachita Mixed Forest-Meadow Ouachita Mountains Lowland Hardwoods': 236,
        'Ouachita Mixed Forest-Meadow Ouachita Mountains Mixed Upland Hardwoods': 237,
        'Ouachita Mixed Forest-Meadow Ouachita Mountains Oak-Hickory': 238,
        'Ouachita Mixed Forest-Meadow Ouachita Mountains Oak-Pine': 239,
        'Ozark Broadleaf Forest-Meadow Boston Mountains Loblolly-Shortleaf': 240,
        'Ozark Broadleaf Forest-Meadow Boston Mountains Lowland Hardwoods': 241,
        'Ozark Broadleaf Forest-Meadow Boston Mountains Mixed Upland Hardwoods': 242,
        'Ozark Broadleaf Forest-Meadow Boston Mountains Oak-Hickory': 243,
        'Ozark Broadleaf Forest-Meadow Boston Mountains Oak-Pine': 244,
        'Prairie Parkland Central Till Plains & Grand Prairies Lowland Hardwoods': 245,
        'Prairie Parkland Central Till Plains & Grand Prairies Maple-Beech-Birch': 246,
        'Prairie Parkland Central Till Plains & Grand Prairies Mixed Upland Hardwoods': 247,
        'Prairie Parkland Central Till Plains & Grand Prairies Oak-Hickory': 248,
        'Prairie Parkland Central Till Plains & Grand Prairies Oak-Pine': 249,
        'Prairie Parkland North Central Plains Lowland Hardwoods': 250,
        'Prairie Parkland North Central Plains Maple-Beech-Birch': 251,
        'Prairie Parkland North Central Plains Mixed Upland Hardwoods': 252,
        'Prairie Parkland North Central Plains Oak-Hickory': 253,
        'Prairie Parkland Red River Valley Lowland Hardwoods': 254,
        'Prairie Parkland Red River Valley Maple-Beech-Birch': 255,
        'Prairie Parkland Red River Valley Oak-Hickory': 256,
        'Puget Trough Mixed Conifer': 257,
        'SE Middle Mixed Forest Arkansas Valley Cypress & Swamp Hardwood': 258,
        'SE Middle Mixed Forest Arkansas Valley Loblolly-Shortleaf': 259,
        'SE Middle Mixed Forest Arkansas Valley Oak-Hickory': 260,
        'SE Middle Mixed Forest Arkansas Valley Oak-Pine': 261,
        'SE Middle Mixed Forest Arkansas Valley Riverine Hardwood': 262,
        'SE Middle Mixed Forest Arkansas Valley Upland Hardwood': 263,
        'SE Middle Mixed Forest Cumberland Plateau & Valley Loblolly-Shortleaf-Oak': 264,
        'SE Middle Mixed Forest Cumberland Plateau & Valley Lowland Hardwood': 265,
        'SE Middle Mixed Forest Cumberland Plateau & Valley Oak-Hickory': 266,
        'SE Middle Mixed Forest Cumberland Plateau & Valley Upland Hardwood': 267,
        'SE Middle Mixed Forest Piedmont Cypress & Swamp Hardwood': 268,
        'SE Middle Mixed Forest Piedmont Loblolly-Shortleaf-Oak': 269,
        'SE Middle Mixed Forest Piedmont Oak-Hickory': 270,
        'SE Middle Mixed Forest Piedmont Riverine Hardwood': 271,
        'SE Middle Mixed Forest Piedmont Upland Hardwood': 272,
        'SE Middle Mixed Forest Western Mid Coastal Plains Cypress & Swamp Hardwood': 273,
        'SE Middle Mixed Forest Western Mid Coastal Plains Loblolly-Shortleaf': 274,
        'SE Middle Mixed Forest Western Mid Coastal Plains Oak-Hickory': 275,
        'SE Middle Mixed Forest Western Mid Coastal Plains Oak-Pine': 276,
        'SE Middle Mixed Forest Western Mid Coastal Plains Riverine Hardwood': 277,
        'SE Middle Mixed Forest Western Mid Coastal Plains Upland Hardwood': 278,
        'Sierra High Elevation Conifer': 279,
        'Sierra Mixed Conifer': 280,
        'Sierra Mixed Oak Woodland': 281,
        'Sierra Pinyon / Juniper Woodland': 282,
        'Sierra Foothill Mixed Oak Woodland': 283,
        'Snake River Basin Mixed Woodland': 284,
        'Alaska Range Transition ': 285,
        'Kodiak Island and Alexander Archipelago ': 286,
        'North Coast Mountains, Chugach-St. Elias Mountains and Gulf of Alaska': 287,
        'Southern Allegheny Plateau Lowland Hardwoods': 288,
        'Southern Allegheny Plateau Mixed Pine-Hardwoods': 289,
        'Southern Allegheny Plateau Northern Conifer': 290,
        'Southern Allegheny Plateau Oak-Hickory': 291,
        'Southern Allegheny Plateau Upland Hardwoods': 292,
        'Southern California Coast Oak Woodlands': 293,
        'Southern California Mixed Conifer': 294,
        'Southern California Oak Woodlands': 295,
        'Southern Cascade High Elevation Conifer': 296,
        'Southern Cascade Mixed Conifer': 297,
        'Southern Cascade Mixed Oak Woodland': 298,
        'Front Range Aspen': 299,
        'Front Range High Elevation Conifer': 300,
        'Front Range Mixed Conifer': 301,
        'Southern Rockies Mixed Conifer': 302,
        'Southern Rockies Pinyon / Juniper Woodland': 303,
        'Southern Rockies Spruce-fir': 304,
        'Southwest High Plains Juniper Mixed Woodland': 305,
        'Southwest High Plains Mesquite Woodland': 306,
        'Southwest High Plains Riparian Hardwoods': 307,
        'Southwest Plateau Hardwoods': 308,
        'Southwest Plateau Juniper Woodland': 309,
        'Southwest Plateau Mesquite': 310,
        'Southwest Plateau Scrub Oak': 311,
        'Southwest Desert Pinyon/Juniper Woodland': 312,
        'Southwestern Rockies Aspen': 313,
        'Southwestern Rockies Mixed Forest': 314,
        'Southwestern Rockies Pinyon / Juniper Woodland': 315,
        'Southwestern Rockies Spruce-fir': 316,
        'St Lawrence & Mohawk Valley Aspen-Birch': 317,
        'St Lawrence & Mohawk Valley Northeast Conifer': 318,
        'St Lawrence & Mohawk Valley Northern Hardwood': 319,
        'Subtropical Prairie Parkland Gulf & Oak Prairie Cypress & Swamp Hardwood': 320,
        'Subtropical Prairie Parkland Gulf & Oak Prairie Oak-Hickory': 321,
        'Subtropical Prairie Parkland Gulf & Oak Prairie Oak-Pine': 322,
        'Subtropical Prairie Parkland Gulf & Oak Prairie Riverine Hardwood': 323,
        'Subtropical Prairie Parkland Gulf & Oak Prairie Upland Hardwood': 324,
        'Subtropical Prairie Parkland Gulf & Oak PrairieMesquite-Juniper Woodlands': 325,
        'Tonto Transition Mixed Conifer': 326,
        'Tonto Transition Oak Woodland': 327,
        'Tonto Transition Pinyon / Juniper Woodland': 328,
        'Tonto Transition Ponderosa Pine': 329,
        'Utah Mountains Aspen': 330,
        'Utah Mountains Mixed Conifer': 331,
        'Utah Mountains Oak Woodlands': 332,
        'Utah Mountains Pinyon / Juniper Woodland': 333,
        'Utah Mountains Spruce-fir': 334,
        'Wasatch Aspen': 335,
        'Wasatch Douglas-fir': 336,
        'Wasatch Oak Woodland': 337,
        'Wasatch Pinyon / Juniper Woodland': 338,
        'Wasatch Spruce / Fir': 339,
        'Western Allegheny Plateau Mixed Hardwood': 340,
        'Western Allegheny Plateau Mixed Pine-Hardwood': 341,
        'Western Allegheny Plateau Oak-Hickory': 342,
        'Western Basin and Range Pinyon/ Juniper Mixed Woodland': 343,
        'Western Great Plains Hardwoods': 344,
        'Western Great Plains Ponderosa Pine': 345,
        'White Mountains Mixed Hardwoods': 346,
        'White Mountains Northeast Spruce-Fir': 347,
        'White Mountains Northern Hardwood': 348,
        'White Mountains Northern Pine Forest': 349,
        'White Mountains - San Francisco Peaks - Mongollon Mixed Conifer': 350,
        'White Mountains - San Francisco Peaks - Mongollon Oak Woodland': 351,
        'White Mountains - San Francisco Peaks - Mongollon Pinyon / Juniper Woodland': 352,
        'White Mountains - San Francisco Peaks - Mongollon Ponderosa Pine': 353,
        'Willamette Woodlands': 354,
        'Yellowstone / Bighorn Douglas-fir / Spruce': 355,
        'Yellowstone / Bighorn Lodgepole': 356,
        'Yellowstone / Bighorn Mixed Woodland': 357,
        'Yellowstone / Bighorn Subalpine Conifer': 358,
    }
